class ManageIQ::Providers::IbmCloud::PowerVirtualServers::NetworkManager < ManageIQ::Providers::NetworkManager
  require_nested :Refresher
  require_nested :CloudNetwork
  require_nested :CloudSubnet
  require_nested :NetworkPort
  require_nested :NetworkRouter
  require_nested :SecurityGroup

  include ManageIQ::Providers::IbmCloud::PowerVirtualServers::ManagerMixin

  delegate :authentication_check,
           :authentication_status,
           :authentication_status_ok?,
           :authentications,
           :authentication_for_summary,
           :zone,
           :connect,
           :verify_credentials,
           :with_provider_connection,
           :address,
           :ip_address,
           :hostname,
           :default_endpoint,
           :endpoints,
           :to        => :parent_manager,
           :allow_nil => true

  def image_name
    "ibm"
  end

  def self.validate_authentication_args(params)
    # return args to be used in raw_connect
    [params[:default_userid], ManageIQ::Password.encrypt(params[:default_password])]
  end

  def self.hostname_required?
    # TODO: ExtManagementSystem is validating this
    false
  end

  def self.ems_type
    @ems_type ||= "ibm_cloud_power_virtual_servers_network".freeze
  end

  def self.description
    @description ||= "IBM Power Systems Virtual Servers Network".freeze
  end

  def create_cloud_subnet(options)
    raw_create_cloud_subnet(self, options)
  end

  def create_cloud_subnet_queue(userid, options = {})
    task_opts = {
      :action => "creating Cloud Subnet for user #{userid}",
      :userid => userid
    }
    queue_opts = {
      :class_name  => self.class.name,
      :method_name => 'create_cloud_subnet',
      :instance_id => id,
      :priority    => MiqQueue::HIGH_PRIORITY,
      :role        => 'ems_operations',
      :zone        => ext_management_system.my_zone,
      :args        => [options]
    }
    MiqTask.generic_action_with_callback(task_opts, queue_opts)
  end

  def raw_create_cloud_subnet(ext_management_system, options)
    ext_management_system.with_provider_connection(:service => 'PowerIaas') do |power_iaas|
      subnet = {
        :type       => options[:type] || 'pub-vlan',
        :name       => options[:name],
        :cidr       => options[:cidr],
        :gateway    => options[:gateway_ip],
        :dnsservers => options[:dns_nameservers],
      }

      power_iaas.create_network(subnet)
    end
  end
end
